{
    'total_results': 1,
    'page': 1,
    'per_page': 30,
    'results': [
        {
            'photos_locked': False,
            'observations_count': 112516,
            'taxon_schemes_count': 1,
            'ancestry': '48460/1/47120/372739/47158/184884/47201/124417/326777/48740/52747',
            'is_active': True,
            'flag_counts': {'unresolved': 0, 'resolved': 0},
            'wikipedia_url': 'http://en.wikipedia.org/wiki/Polistinae',
            'current_synonymous_taxon_ids': None,
            'iconic_taxon_id': 47158,
            'taxon_photos': [
                {
                    'taxon_id': 343248,
                    'photo': {
                        'flags': [],
                        'type': 'LocalPhoto',
                        'url': 'https://static.inaturalist.org/photos/22765568/square.jpeg?1533696452',
                        'square_url': 'https://static.inaturalist.org/photos/22765568/square.jpeg?1533696452',
                        'native_page_url': 'https://www.inaturalist.org/photos/22765568',
                        'native_photo_id': '22765568',
                        'small_url': 'https://static.inaturalist.org/photos/22765568/small.jpeg?1533696452',
                        'original_url': 'https://static.inaturalist.org/photos/22765568/original.jpeg?1533696452',
                        'attribution': '(c) copper, some rights reserved (CC BY-NC)',
                        'medium_url': 'https://static.inaturalist.org/photos/22765568/medium.jpeg?1533696452',
                        'id': 22765568,
                        'license_code': 'cc-by-nc',
                        'original_dimensions': {'width': 2048, 'height': 1639},
                        'large_url': 'https://static.inaturalist.org/photos/22765568/large.jpeg?1533696452',
                    },
                    'taxon': {
                        'photos_locked': False,
                        'taxon_schemes_count': 1,
                        'ancestry': '48460/1/47120/372739/47158/184884/47201/124417/326777/48740/52747',
                        'min_species_ancestry': '48460,1,47120,372739,47158,184884,47201,124417,326777,48740,52747,343248',
                        'wikipedia_url': 'http://en.wikipedia.org/wiki/Polistinae',
                        'current_synonymous_taxon_ids': None,
                        'iconic_taxon_id': 47158,
                        'created_at': '2014-02-19T09:17:40+00:00',
                        'taxon_changes_count': 0,
                        'complete_species_count': None,
                        'rank': 'subfamily',
                        'extinct': False,
                        'id': 343248,
                        'universal_search_rank': 112516,
                        'ancestor_ids': [
                            48460,
                            1,
                            47120,
                            372739,
                            47158,
                            184884,
                            47201,
                            124417,
                            326777,
                            48740,
                            52747,
                            343248,
                        ],
                        'observations_count': 112516,
                        'is_active': True,
                        'flag_counts': {'unresolved': 0, 'resolved': 0},
                        'min_species_taxon_id': 343248,
                        'rank_level': 27,
                        'atlas_id': None,
                        'parent_id': 52747,
                        'name': 'Polistinae',
                        'default_photo': {
                            'square_url': 'https://static.inaturalist.org/photos/22765568/square.jpeg?1533696452',
                            'attribution': '(c) copper, some rights reserved (CC BY-NC)',
                            'flags': [],
                            'medium_url': 'https://static.inaturalist.org/photos/22765568/medium.jpeg?1533696452',
                            'id': 22765568,
                            'license_code': 'cc-by-nc',
                            'original_dimensions': {'width': 2048, 'height': 1639},
                            'url': 'https://static.inaturalist.org/photos/22765568/square.jpeg?1533696452',
                        },
                        'iconic_taxon_name': 'Insecta',
                        'preferred_common_name': 'Paper Wasps',
                    },
                },
            ],
            'rank_level': 27,
            'taxon_changes_count': 0,
            'atlas_id': None,
            'complete_species_count': None,
            'parent_id': 52747,
            'name': 'Polistinae',
            'rank': 'subfamily',
            'extinct': False,
            'id': 343248,
            'default_photo': {
                'square_url': 'https://static.inaturalist.org/photos/22765568/square.jpeg?1533696452',
                'attribution': '(c) copper, some rights reserved (CC BY-NC)',
                'flags': [],
                'medium_url': 'https://static.inaturalist.org/photos/22765568/medium.jpeg?1533696452',
                'id': 22765568,
                'license_code': 'cc-by-nc',
                'original_dimensions': {'width': 2048, 'height': 1639},
                'url': 'https://static.inaturalist.org/photos/22765568/square.jpeg?1533696452',
            },
            'ancestor_ids': [
                48460,
                1,
                47120,
                372739,
                47158,
                184884,
                47201,
                124417,
                326777,
                48740,
                52747,
            ],
            'iconic_taxon_name': 'Insecta',
            'preferred_common_name': 'Paper Wasps',
            'ancestors': [
                {
                    'observations_count': 32873107,
                    'taxon_schemes_count': 2,
                    'ancestry': '48460',
                    'is_active': True,
                    'flag_counts': {'unresolved': 0, 'resolved': 9},
                    'wikipedia_url': 'http://en.wikipedia.org/wiki/Animal',
                    'current_synonymous_taxon_ids': None,
                    'iconic_taxon_id': 1,
                    'rank_level': 70,
                    'taxon_changes_count': 4,
                    'atlas_id': None,
                    'complete_species_count': None,
                    'parent_id': 48460,
                    'complete_rank': 'order',
                    'name': 'Animalia',
                    'rank': 'kingdom',
                    'extinct': False,
                    'id': 1,
                    'default_photo': {
                        'square_url': 'https://static.inaturalist.org/photos/80678745/square.jpg?1593020613',
                        'attribution': 'Ningún derecho reservado',
                        'flags': [],
                        'medium_url': 'https://static.inaturalist.org/photos/80678745/medium.jpg?1593020613',
                        'id': 80678745,
                        'license_code': 'cc0',
                        'original_dimensions': {'width': 2000, 'height': 2000},
                        'url': 'https://static.inaturalist.org/photos/80678745/square.jpg?1593020613',
                    },
                    'ancestor_ids': [48460, 1],
                    'iconic_taxon_name': 'Animalia',
                    'preferred_common_name': 'Animals',
                },
            ],
            'children': [
                {
                    'observations_count': 88534,
                    'taxon_schemes_count': 0,
                    'ancestry': '48460/1/47120/372739/47158/184884/47201/124417/326777/48740/52747/343248',
                    'is_active': True,
                    'flag_counts': {'unresolved': 0, 'resolved': 0},
                    'wikipedia_url': 'http://en.wikipedia.org/wiki/Polistes',
                    'current_synonymous_taxon_ids': None,
                    'iconic_taxon_id': 47158,
                    'rank_level': 25,
                    'taxon_changes_count': 0,
                    'atlas_id': None,
                    'complete_species_count': None,
                    'parent_id': 343248,
                    'name': 'Polistini',
                    'rank': 'tribe',
                    'extinct': False,
                    'id': 638567,
                    'default_photo': {
                        'square_url': 'https://static.inaturalist.org/photos/2367678/square.jpg?1441941708',
                        'attribution': '(c) Cheryl Harleston López Espino, some rights reserved (CC BY-NC-ND)',
                        'flags': [],
                        'medium_url': 'https://static.inaturalist.org/photos/2367678/medium.jpg?1441941708',
                        'id': 2367678,
                        'license_code': 'cc-by-nc-nd',
                        'original_dimensions': {'width': 900, 'height': 1200},
                        'url': 'https://static.inaturalist.org/photos/2367678/square.jpg?1441941708',
                    },
                    'ancestor_ids': [
                        48460,
                        1,
                        47120,
                        372739,
                        47158,
                        184884,
                        47201,
                        124417,
                        326777,
                        48740,
                        52747,
                        343248,
                        638567,
                    ],
                    'iconic_taxon_name': 'Insecta',
                },
            ],
            'conservation_statuses': [],
            'listed_taxa_count': 0,
            'listed_taxa': [],
            'wikipedia_summary': 'The <b>Polistinae</b> are eusocial wasps closely related to the more familiar yellow jackets, but placed in their own subfamily, containing four tribes; with some 1100 species total, it is the second-most diverse subfamily within the Vespidae, and while most species are tropical or subtropical, they include some of the most frequently encountered large wasps in temperate regions. They are also known as paper wasps, which is a misleading term since other wasps (including yellow jackets) also...',
        }
    ],
}
