from datetime import datetime
from dateutil.tz import tzutc

{
    datetime(2020, 1, 1, 14, 0, tzinfo=tzutc()): 1,
    datetime(2020, 1, 1, 15, 0, tzinfo=tzutc()): 13,
    datetime(2020, 1, 1, 16, 0, tzinfo=tzutc()): 0,
    datetime(2020, 1, 1, 17, 0, tzinfo=tzutc()): 0,
    datetime(2020, 1, 1, 18, 0, tzinfo=tzutc()): 0,
    datetime(2020, 1, 1, 19, 0, tzinfo=tzutc()): 3,
    datetime(2020, 1, 1, 20, 0, tzinfo=tzutc()): 2,
    datetime(2020, 1, 1, 21, 0, tzinfo=tzutc()): 1,
    datetime(2020, 1, 1, 22, 0, tzinfo=tzutc()): 1,
    datetime(2020, 1, 1, 23, 0, tzinfo=tzutc()): 0,
    datetime(2020, 1, 2, 0, 0, tzinfo=tzutc()): 1,
    datetime(2020, 1, 2, 1, 0, tzinfo=tzutc()): 0,
    datetime(2020, 1, 2, 2, 0, tzinfo=tzutc()): 0,
    datetime(2020, 1, 2, 3, 0, tzinfo=tzutc()): 1,
}
