[
    "   52747:       Family Vespidae (Hornets, Paper Wasps, Potter Wasps, and Allies)",
    "   84738:    Subfamily Vespinae (Hornets and Yellowjackets)",
    "  131878:      Species Nicrophorus vespillo (Vespillo Burying Beetle)",
    "  621585:      Species Vespicula trachinoides (Vespicula Waspfish)",
    "  495392:      Species Vespidae st1",
    "   70118:      Species Nicrophorus vespilloides (Lesser Vespillo Burying Beetle)",
    "   92786:        Genus Vespicula",
    "  646195:        Genus Vespiodes",
    "  621584:      Species Vespicula cypho",
    "  621586:      Species Vespicula zollingeri",
]
