/* Add contrast to function signatures, similar to sphinx-rtd-theme */
.sig-object {
    background-color: #ecf0f3;
    display: table;
    border-top: 3px solid #00bfa5;
    padding: 5px;
}

.sig-prename {
    color: #546e7a;
}

.sig-name {
    color: #222;
    font-weight: 600;
}

dt:target {
    margin-top: 0;
    padding-top: 0;
}