from datetime import datetime
from dateutil.tz import tzutc

{
    'total_results': 3,
    'page': 1,
    'per_page': 3,
    'results': [
        {
            'id': 886482,
            'login': 'niconoe',
            'spam': False,
            'suspended': False,
            'created_at': datetime(2018, 4, 23, 17, 11, 14, tzinfo=tzutc()),
            'login_autocomplete': 'niconoe',
            'login_exact': 'niconoe',
            'name': 'Nicolas Noé',
            'name_autocomplete': 'Nicolas Noé',
            'orcid': 'https://orcid.org/0000-0002-9503-4750',
            'icon': 'https://static.inaturalist.org/attachments/users/icons/886482/thumb.jpg?1529671435',
            'observations_count': 928,
            'identifications_count': 118,
            'journal_posts_count': 0,
            'activity_count': 1046,
            'species_count': 396,
            'universal_search_rank': 928,
            'roles': [],
            'site_id': 1,
            'icon_url': 'https://static.inaturalist.org/attachments/users/icons/886482/medium.jpg?1529671435',
        },
        {
            'id': 2909130,
            'login': 'niconoerbo',
            'spam': False,
            'suspended': False,
            'created_at': datetime(2020, 5, 5, 6, 28, 32, tzinfo=tzutc()),
            'login_autocomplete': 'niconoerbo',
            'login_exact': 'niconoerbo',
            'name': None,
            'name_autocomplete': None,
            'orcid': None,
            'icon': None,
            'observations_count': 6,
            'identifications_count': 0,
            'journal_posts_count': 0,
            'activity_count': 6,
            'universal_search_rank': 6,
            'roles': [],
            'site_id': 1,
            'icon_url': None,
        },
        {
            'id': 3358478,
            'login': 'nicono',
            'spam': False,
            'suspended': False,
            'created_at': datetime(2020, 7, 20, 18, 7, 44, tzinfo=tzutc()),
            'login_autocomplete': 'nicono',
            'login_exact': 'nicono',
            'name': None,
            'name_autocomplete': None,
            'orcid': None,
            'icon': None,
            'observations_count': 0,
            'identifications_count': 0,
            'journal_posts_count': 0,
            'activity_count': 0,
            'universal_search_rank': 0,
            'roles': [],
            'site_id': 1,
            'icon_url': None,
        },
    ],
}
